/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;

/**
 * The EJB connection pool. EJB are stored using their provides configuration
 * as key. Some additional informations such as the EJB configuration, the
 * classloader used are stored whitin the EjbConnectionPoolElement instances.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbConnectionPool {

    /**
     * Logger used within this class.
     */
    private final Logger logger;

    /**
     * The EJB connection pool. The EJB can be retrieve by its Provides
     * configuration.
     */
    private Map<Provides, EjbConnectionPoolElement> ejbPool;

    /**
     * Default constructor.
     *
     * @param logger
     *      the logger used to log informations from this class
     */
    public EjbConnectionPool(final Logger logger) {
        this.logger = logger;
        this.ejbPool = new HashMap<Provides, EjbConnectionPoolElement>();
    }

    /**
     * Adds an EJB to the connection pool.
     *
     * @param provides
     *      the key used to store the EJB
     * @param poolElement
     *      the EJB with some additional informations
     * @throws org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException
     *      if the EJB already registered whitin an instance of this pool
     */
    public void addEjb(final Provides provides, final EjbConnectionPoolElement poolElement) throws EjbConnectionPoolException {
        //If tbe EJB already exists in the connection pool
        if (this.ejbPool.containsKey(provides)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("EJB : ");
            errorSB.append(poolElement.getEjbConfiguration().toString());
            errorSB.append(" already registrered.");
            throw new EjbConnectionPoolException(errorSB.toString());
        }

        this.ejbPool.put(provides, poolElement);
    }

    /**
     * Retrieve an EJB from the pool using its Provides configuration.
     *
     * @param provides
     *      the provides configuration of the EJB
     * @return
     *      the EJB with some additional informations
     * @throws org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException
     *      if the EJB is not registered
     */
    public EjbConnectionPoolElement getEjb(final Provides provides) throws EjbConnectionPoolException {

        //If the EJB has not been added to the pool
        if (! this.ejbPool.containsKey(provides)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't find EJB for provides : ");
            errorSB.append(provides.toString());
            throw new EjbConnectionPoolException(errorSB.toString());
        }

        //Retireve the value
        EjbConnectionPoolElement poolElement = this.ejbPool.get(provides);

        //If the value of the EjbPoolValue is null
        if (poolElement == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't find EJB for provides : ");
            errorSB.append(provides);
            throw new EjbConnectionPoolException(errorSB.toString());
        }

        return poolElement;
    }

    /**
     * Remove the EJB from the pool using its Provides configuration.
     *
     * @param provides
     *      the provides configuration of the EJB
     * @throws org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException
     *      if the EJB is not registered
     */
    public void removeEjb(final Provides provides) throws EjbConnectionPoolException {
        //If the EJB has not been added to the pool
        if (! this.ejbPool.containsKey(provides)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't find EJB for provides : ");
            errorSB.append(provides);
            throw new EjbConnectionPoolException(errorSB.toString());
        }

        //Remove the entry
        this.ejbPool.remove(provides);
    }

}
