    /**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.configuration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.ow2.petals.bc.ejb.exceptions.ConfigurationException;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Holds the configuration of a given EJB.
 *
 * NOTE : all the fields in this class are final because the validation of the
 * given configuration is done in the constructor.
 *
 * NOTE : having fields that are not final have ne sense here, because theses
 * informations are not supposed to change during the live of an instance of
 * this very class.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbConfiguration {
/**
     * List of the configuration elements.
     */
    private Map<String, ConfigurationElement> ejbConfigurationElements;

    /**
     * Default constructor.
     *
     * @param ejbJndiName
     *      the EJB JNDI name
     * @param ejbHomeInterface
     *      the EJB (remote) home interface (for EJB2.x)
     * @param ejbVersion
     *      the EJB version (must be one of 2.0, 2.1, 3.0, 3.1)
     * @param javaNamingFactoryInitial
     *      the java naming initial context factory
     * @param javaNamingProviderUrl
     *      the java naming provider url
     * @param securityName
     *      the security name to use (if defined the class must be in the SU or
     *      in the component)
     * @param securityPrincipal
     *      the "user"
     * @param securityCredentials
     *      the "password"
     * @param dataBindingEngine
     *      the databinding engine (still there for backward compatibility reasons)
     * @param wsdlFileName
     *      the WSDL file name
     * @param serviceName
     *      the service name
     * @throws org.ow2.petals.bc.ejb.exceptions.ConfigurationException
     *      if the given configuration is not valid
     */
    public EjbConfiguration(
            final String ejbJndiName,
            final String ejbHomeInterface,
            final String ejbVersion,
            final String javaNamingFactoryInitial,
            final String javaNamingFactoryUrlPkgs,
            final String javaNamingProviderUrl,
            final String securityName,
            final String securityPrincipal,
            final String securityCredentials,
            final String dataBindingEngine,
            final String wsdlFileName,
            final String serviceName) throws ConfigurationException {

        //Creates the configuration elements list
        this.ejbConfigurationElements = new HashMap<String, ConfigurationElement>();

        //Informations on the EJB
        EjbElement ejbElement = new EjbElement(ejbJndiName, ejbHomeInterface, ejbVersion);
        ejbConfigurationElements.put(EjbElement.CONFIGURATION_ELEMENT_NAME, ejbElement);

        //JNDI informations
        AppServerElement appServerElement = new AppServerElement(javaNamingFactoryInitial, javaNamingFactoryUrlPkgs, javaNamingProviderUrl);
        ejbConfigurationElements.put(AppServerElement.CONFIGURATION_ELEMENT_NAME, appServerElement);

        //Security informations
        SecurityElement securityElement = null;

        //If we have a security element
        if (! StringHelper.isNullOrEmpty(securityName)) {
            securityElement = new SecurityElement();
            securityElement.setSecurityName(securityName);
        }

        //If we have a login or a password
        if ((! StringHelper.isNullOrEmpty(securityPrincipal)) || (! StringHelper.isNullOrEmpty(securityCredentials))) {
            securityElement = (securityElement != null)?securityElement:new SecurityElement();
            securityElement.setSecurityPrincipal(securityPrincipal);
            securityElement.setSecurityCredentials(securityCredentials);
        }

        //If a security element has been created
        if (securityElement != null) {
            ejbConfigurationElements.put(SecurityElement.CONFIGURATION_ELEMENT_NAME, securityElement);
        }

        //Informations for the databinding module
        DatabindingElement databindingElement = new DatabindingElement(dataBindingEngine, wsdlFileName, serviceName);
        ejbConfigurationElements.put(DatabindingElement.CONFIGURATION_ELEMENT_NAME, databindingElement);

        //Validates the current configuration
        this.validate();
    }

    /**
     * Accessor on the informations relative to the EJB.
     *
     * @return
     *      an EjbElement object holding some informations on the EJB
     * @throws ConfigurationException
     *      if the element can't be retrieved
     */
    public EjbElement getEjbElement() throws ConfigurationException {
        //Retrieve the configuration element
        Object supposedEjbElement = this.ejbConfigurationElements.get(EjbElement.CONFIGURATION_ELEMENT_NAME);

        //Check if the cast is possible
        if ((supposedEjbElement != null) && (supposedEjbElement instanceof EjbElement)) {
            return (EjbElement) supposedEjbElement;
        }

        //Else oups Houston we have a problem
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't retrieve the EjbElement from the EjbConnectionInfo instance.");
        throw new ConfigurationException(errorSB.toString());
    }

    /**
     * Accessor on the informations relative the JNDI of the application server.
     *
     * @return
     *      an AppServerElement object holding some informations on the JNDI of
     *      the application server.
     * @throws ConfigurationException
     *      if the element can't be retrieved
     */
    public AppServerElement getAppServerElement() throws ConfigurationException {
        //Retrieve the configuration element
        Object supposedAppServerElement = this.ejbConfigurationElements.get(AppServerElement.CONFIGURATION_ELEMENT_NAME);

        //Check if the cast is possible
        if ((supposedAppServerElement != null) && (supposedAppServerElement instanceof AppServerElement)) {
            return (AppServerElement) supposedAppServerElement;
        }

        //Else oups Houston we have a problem
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't retrieve the AppServerElement from the EjbConnectionInfo instance.");
        throw new ConfigurationException(errorSB.toString());
    }

    /**
     * Accessor on the information relative to the security of the application
     * server.
     *
     * @return
     *      a SecurityElement object that holds some informations on the security
     *      configuration of the application server or null if no security has
     *      been defined
     * @throws ConfigurationException
     *      if the element can't be retrieved
     */
    public SecurityElement getSecurityElement() throws ConfigurationException {
        if (this.hasSecurityElement()) {

            //Retrieve the configuration element
            Object supposedSecurityElement = this.ejbConfigurationElements.get(SecurityElement.CONFIGURATION_ELEMENT_NAME);

            //Check if the cast is possible
            if ((supposedSecurityElement != null) && (supposedSecurityElement instanceof SecurityElement)) {
                return (SecurityElement) supposedSecurityElement;
            }

            //Else oups Houston we have a problem
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the SecurityElement from the EJB configuration.");
            throw new ConfigurationException(errorSB.toString());
        }

        return null;
    }

    /**
     * Accessor on the databinding informations.
     *
     * @return
     *      a DatabindingElement object that holds some informations required by
     *      the databinding engine.
     * @throws ConfigurationException
     *      if the element can't be retrieved
     */
    public DatabindingElement getDatabindingElement() throws ConfigurationException {
        //Retrieve the configuration element
        Object supposedDatabindingElement = this.ejbConfigurationElements.get(DatabindingElement.CONFIGURATION_ELEMENT_NAME);

        //Check if the cast is possible
        if (supposedDatabindingElement instanceof DatabindingElement) {
            return (DatabindingElement) supposedDatabindingElement;
        }

        //Else oups Houston we have a problem
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't retrieve the DatabindingElement from the EJB configuration.");
        throw new ConfigurationException(errorSB.toString());
    }

    /**
     * Tests whether or not a security element has been found.
     *
     * @return true if a SecurityElement has been found
     */
    public boolean hasSecurityElement() {
        //If the no security configuration has been inserted
        if (! this.ejbConfigurationElements.containsKey(SecurityElement.CONFIGURATION_ELEMENT_NAME)) {
            return false;
        }

        //Retrieve the configuration element
        Object supposedSecurityElement = this.ejbConfigurationElements.get(SecurityElement.CONFIGURATION_ELEMENT_NAME);

        //If the retrieved securityElement is null
        if (supposedSecurityElement == null) {
            return false;
        }

        //Check if the securityElement if of the good type
        if (! (supposedSecurityElement instanceof SecurityElement)) {
            return false;
        }

        return true;
    }

    /**
     * Validates the current configuration. If the configuration is not valid,
     * an exception is thrown. The message associated to this exception spot
     * every encountered configuration issues.
     *
     * @throws org.ow2.petals.bc.ejb.exceptions.ConfigurationException
     *      raised if one or more configuration issue has been found
     */
    public void validate() throws ConfigurationException {
        //Holds the error message
        StringBuilder errorSB = new StringBuilder();
        StringBuilder missingElementsSB = new StringBuilder("missing configuration element(s) : ");
        StringBuilder invalidElementsSB = new StringBuilder("invalid configuration element(s) : ");

        //To "stream" the missing values
        List<String> missingElements = new ArrayList<String>();

        //To stream the invalid elements
        List<String> invalidElements = new ArrayList<String>();

        //Retrieve all the missing & invalid values
        for (Entry<String, ConfigurationElement> entry : this.ejbConfigurationElements.entrySet()) {

            //Adds the missing configuration artefacts to the list
            missingElements.addAll(entry.getValue().findMissingElements());

            //Adds the invalid configuration artefacts to the list
            invalidElements.addAll(entry.getValue().findInvalidElements());
        }

        //Loops on the missing elements & adds them to the StringBuilder
        for (String missingElement : missingElements) {
            missingElementsSB.append(missingElement + ", ");
        }

        //Loops on the invalid elements & adds them to the StringBuilder
        for (String invalidElement : invalidElements) {
            invalidElementsSB.append(invalidElement + ", ");
        }

        //If one of the two (inavalid or missing)Elements list is not empty
        if (! (missingElements.isEmpty() && invalidElements.isEmpty())) {

            errorSB.append("Can't create ConfigurationExtensions from the given ");
            errorSB.append("JBI descriptor content : ");

            //If there are missing elements
            if (! missingElements.isEmpty()) {
                errorSB.append(missingElementsSB.toString());
            }

            //If there are invalid elements
            if (! invalidElements.isEmpty()) {
                errorSB.append(invalidElementsSB.toString());
            }

            //Raises an exception
            throw new ConfigurationException(errorSB.toString());
        }
    }

    @Override
    public String toString() {
        StringBuilder toStringSB = new StringBuilder();

        for (Entry<String, ConfigurationElement> entry : this.ejbConfigurationElements.entrySet()) {
            toStringSB.append(entry.getKey() + " : ");
            toStringSB.append(entry.getValue().toString());
            toStringSB.append("\n");
        }

        return toStringSB.toString();
    }
}
