/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.configuration;

import java.util.ArrayList;
import java.util.List;
import org.ow2.petals.bc.ejb.constants.SupportedEjbVersions;
import org.ow2.petals.bc.ejb.constants.ProvidesConstants;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Holds the informations on an EJB like its name, its version, and where to
 * find it in the JNDI of the application server.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbElement implements ConfigurationElement {

    public static final String CONFIGURATION_ELEMENT_NAME = "info";

    /**
     * JNDI name of the EJB
     */
    private final String ejbJndiName;

    /**
     * Fully qualified name of the (remote) home interface of the EJB
     */
    private final String ejbHomeInterface;

    /**
     * Version of the EJB
     */
    private final String ejbVersion;

    /**
     * Default constructor.
     *
     * @param ejbJndiName
     *      the jndi name of the EJB
     * @param ejbHomeInterface
     *      the remote home inferface of the EJB (2.x)
     * @param ejbVersion
     *      the EJB version
     */
    public EjbElement(final String ejbJndiName,
            final String ejbHomeInterface,
            final String ejbVersion) {

        //Sets the values
        this.ejbJndiName = ejbJndiName;
        this.ejbHomeInterface = ejbHomeInterface;
        this.ejbVersion = ejbVersion;

    }

    /**
     * Accessor on the EJB JNDI name.
     *
     * @return the EJB JNDI name
     */
    public final String getEjbJndiName() {
        return ejbJndiName;
    }

    /**
     * Accessor on the EJB home interface.
     *
     * @return the fully qualified name of the home interface
     */
    public final String getEjbHomeInterface() {
        return ejbHomeInterface;
    }

    /**
     * Accessor on the EJB version.
     *
     * @return the EJB version
     */
    public final String getEjbVersion() {
        return ejbVersion;
    }

    /**
     * Checks if the EJB_VERSION is 2.0 or 2.1.
     *
     * @return true if EJB_VERSION is 2.0 or 2.1
     */
    public final boolean isEJB2x() {
        //If a valid EJB version is defined
        if (this.hasEjbVersion() && this.hasValidEjbVersion()) {
            //Checks for EJB 2.0
            if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_0)) {
                return true;
            }

            //Checks for EJB 2.1
            if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_1)) {
                return true;
            }

            //EJB version is 3.x
            return false;
        }

        return false;
    }

    /**
     * Checks if the EJB_VERSION is 3.0 or 3.1.
     *
     * @return true if EJB_VERSION is 3.0 or 3.1
     */
    public final boolean isEJB3x() {

        //If a valid EJB version is defined
        if (this.hasEjbVersion() && this.hasValidEjbVersion()) {
            //Checks for EJB 3.0
            if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_0)) {
                return true;
            }

            //Checks for EJB 3.1
            if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_1)) {
                return true;
            }

            //EJB version is 2.x
            return false;
        }

        return false;
    }

    /**
     * Retrieve all the missing configuration parameters for the current
     * configuration element. The missing configuration elements are returned
     * as a List.
     *
     * @return a list of the missing elements
     */
    public List<String> findMissingElements() {
        List<String> missingElements = new ArrayList<String>();

        //Check for the JNDI name
        if (! this.hasEjbJndiName()) {
            missingElements.add(ProvidesConstants.EJB_JNDI_NAME);
        }

        //Checks for the EJB version
        if (! this.hasEjbVersion()) {
            missingElements.add(ProvidesConstants.EJB_VERSION);
        }

        //Check for the need of the home interface
        if (this.isEJB2x()) {
            //If the EJB version is 2.0 or 2.1 a home interface is expeted
            if (! this.hasEjbHomeInterface()) {
                missingElements.add(ProvidesConstants.EJB_HOME_INTERFACE);
            }
        }

        return missingElements;
    }

    /**
     * Retrieve all the invalid configuration parameters for the current
     * configuration element. The invalid configuration elements are returned
     * as a List.
     *
     * @return a list of the invalid elements
     */
    public List<String> findInvalidElements() {
        List<String> invalidElements = new ArrayList<String>();

        //Checks for the EJB version
        if (this.hasEjbVersion()) {
            if (! this.hasValidEjbVersion()) {
                invalidElements.add(ProvidesConstants.EJB_VERSION);
            }
        }

        return invalidElements;
    }

    /**
     * Return a String description of the current instance.
     *
     * @return
     *      a String description of the current instance
     */
    @Override
    public String toString() {
        StringBuilder toStringSB = new StringBuilder();
        toStringSB.append("ejbJndiName=" + this.ejbJndiName);
        toStringSB.append(", ");
        toStringSB.append("ejbHomeInterface=" + this.ejbHomeInterface);
        toStringSB.append(", ");
        toStringSB.append("ejbVersion=" + this.ejbVersion);

        return toStringSB.toString();
    }

    /**
     * Checks if the field EJB_JNDI_NAME is not null or empty.
     *
     * @return true if field is valid
     */
    private final boolean hasEjbJndiName() {

        if (StringHelper.isNullOrEmpty(this.ejbJndiName)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if the field EJB_HOME_INTERFACE is not null or empty.
     *
     * @return true if the field is not null or empty
     */
    private final boolean hasEjbHomeInterface() {

        if (StringHelper.isNullOrEmpty(this.ejbHomeInterface)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if the field EJB_VERSION is not null or empty.
     *
     * @return true if the field is not null or empty
     */
    private final boolean hasEjbVersion() {

        //Checks if the field is not empty or null
        if (StringHelper.isNullOrEmpty(this.ejbVersion)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if the content of the field EJB_VERSION is valid or not.
     *
     * @return true if the field is valid
     */
    private final boolean hasValidEjbVersion() {

        if (this.hasEjbVersion()) {
            //Checks if the content is valid or not
            if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_0)) {
                return true;
            } else if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_1)) {
                return true;
            } else if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_0)) {
                return true;
            } else if (this.ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_1)) {
                return true;
            }
        }

        return false;
    }
}
