/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.service;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;
import javax.rmi.PortableRemoteObject;
import org.ow2.petals.bc.ejb.configuration.EjbConfiguration;
import org.ow2.petals.bc.ejb.configuration.SecurityElement;
import org.ow2.petals.bc.ejb.constants.SupportedEjbVersions;
import org.ow2.petals.bc.ejb.exceptions.ConfigurationException;
import org.ow2.petals.bc.ejb.exceptions.ConnectionException;
import org.ow2.petals.bc.ejb.util.EJbMethodsDisplayer;

/**
 * Utility class that performs most of the operation required to retrieve
 * an EJB from the application server.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbConnection {

    /**
     * A logger.
     */
    private Logger logger;

    /**
     * Information on the EJB, the databinding engine & the AppServer.
     */
    private EjbConfiguration info;

    /**
     * The classloader to use.
     */
    private ClassLoader classLoader = null;

    /**
     * Default constructor.
     *
     * @param info
     * @param classLoader
     * @param logger
     */
    public EjbConnection(EjbConfiguration info, ClassLoader classLoader, final Logger logger) {
        super();
        this.info = info;
        this.logger = Logger.getLogger(logger.getName() + ".EjbConnection");
        this.classLoader = classLoader;
    }

    /**
     * Retrieve the EJB object using the given configuration, from the app
     * server.
     *
     * @param info
     *      informations on the EJB
     * @return
     *      the EJB object
     * @throws org.ow2.petals.bc.ejb.exceptions.ConnectionException
     *      if a connection can't be opened to the app server
     * @throws org.ow2.petals.bc.ejb.exceptions.ConfigurationException
     *      if the given configuration is not valid or if any of the element
     *      of the configuration can't be retrieved for any reason
     */
    public Object retrieveObject(final EjbConfiguration info) throws ConnectionException, ConfigurationException {

        //The EJB object
        Object ejb = null;

        //Creates a new initial context instance
        final Context initialContext = this.createInitialContext();

        //The EJB version
        final String ejbVersion = info.getEjbElement().getEjbVersion();

        //If the EJB version is 2.0 or 2.1
        if (ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_0) || ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_2_1)) {
            try {
                //Try to retrieve an EJB 2.x from the context
                Object ref = initialContext.lookup(info.getEjbElement().getEjbJndiName());

                //Retrieve the remote interface of the EJB
                Class<?> homeInterface = loadEjbHomeClass(info);

                //Bind the retrieved interface
                Object home = PortableRemoteObject.narrow(ref, homeInterface);

                //Call the create method of the EJB
                ejb = createEjb2x(home);
            } catch (NamingException ex) {
                //Build the error message
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve EJB 2.x for name : ");
                errorSB.append(info.getEjbElement().getEjbJndiName());
                errorSB.append(" from the given context : ");
                errorSB.append(initialContext.toString());

                //Raise an exception
                throw new ConnectionException(errorSB.toString(), ex);
            }
        } //If the EJB version is 3.0 or 3.1
        else if (ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_0) || ejbVersion.equals(SupportedEjbVersions.EJB_VERSION_3_1)) {
            //Try to retrieve an EJB 3.x from the given context
            try {
                ejb = initialContext.lookup(info.getEjbElement().getEjbJndiName());
            } catch (NamingException ex) {
                //Build the error message
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve EJB 3.x for name : ");
                errorSB.append(info.getEjbElement().getEjbJndiName());
                errorSB.append(" from the given context : ");
                errorSB.append(initialContext.toString());

                //Raise an exception
                throw new ConnectionException(errorSB.toString(), ex);
            }
        } //Else the EJB version is not supported : should never happen
        else {
            throw new ConnectionException("Unknown EJB version");
        }

        StringBuilder logMsgBuilder = new StringBuilder();
        logMsgBuilder.append("Lookup successfully the EJB : ");
        logMsgBuilder.append(info.getEjbElement().getEjbJndiName());
        logMsgBuilder.append(" (EJB class : " + ejb.getClass().getName() + ")");

        this.logger.log(Level.INFO, logMsgBuilder.toString());

        //EJbMethodsDisplayer.displayMethodsForClass(ejb.getClass());

        return ejb;
    }

    /**
     * Create a new initial context instance using the given informations.
     *
     * @return a new initial context
     * @throws org.ow2.petals.bc.ejb.exceptions.ConnectionException
     * @throws ConfigurationException
     */
    public Context createInitialContext() throws ConnectionException, ConfigurationException {
        //Properties used to store the ones required to instanciate the initial context factory
        Properties environment = new Properties();

        //Put the required information in the properties
        environment.put(Context.PROVIDER_URL, this.info.getAppServerElement().getJavaNamingProviderUrl());
        environment.put(Context.INITIAL_CONTEXT_FACTORY, this.info.getAppServerElement().getJavaNamingFactoryInitial());

        if (this.info.getAppServerElement().hasJavaNamingFactoryUrlPkgs()) {
            environment.put(Context.URL_PKG_PREFIXES, this.info.getAppServerElement().getJavaNamingFactoryUrlPkgs());
        }

        //For the security related informations
        if (info.hasSecurityElement()) {
            final SecurityElement securityElement = this.info.getSecurityElement();

            final String securityPrincipal = securityElement.getSecurityPrincipal();
            final String securityCredentials = securityElement.getSecurityCredentials();

            if (!((securityPrincipal == null) || (securityPrincipal.equals("")))) {
                if (!((securityCredentials == null) || (securityCredentials.equals("")))) {
                    environment.put(Context.SECURITY_PRINCIPAL, securityPrincipal);
                    environment.put(Context.SECURITY_CREDENTIALS, securityCredentials);
                }
            }
        }

        //Creates a new initial context from the above properties
        return createJNDIContext(environment);
    }

    /**
     * Creates a new initial context using the given (context) properties.
     *
     * @param properties (context) properties
     * @return a new initial context instance
     * @throws ConnectionException
     */
    private Context createJNDIContext(final Properties properties) throws ConnectionException {
        //Used to store the retrieved initial context
        Context initialContext = null;

        //Initial Context factory raw class
        Class<?> initialContextFactoryClass = null;

        //An initial context factory instance
        InitialContextFactory initialContextFactoryInstance = null;

        try {
            //Try to load the initial context factory
            initialContextFactoryClass = classLoader.loadClass(properties.getProperty(Context.INITIAL_CONTEXT_FACTORY));

            //Try to get an instance of the initial context factory class
            initialContextFactoryInstance = (InitialContextFactory) initialContextFactoryClass.newInstance();

            //Try to get an initial context from the initial context factory
            initialContext = initialContextFactoryInstance.getInitialContext(properties);
        } catch (NamingException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create initial context : ");
            errorSB.append("can't get initial context from the initial context factory instance : ");
            errorSB.append(properties.getProperty(Context.INITIAL_CONTEXT_FACTORY));
            errorSB.append(" : NamingException.");

            //Raise an exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (InstantiationException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create initial context : ");
            errorSB.append("can't instanciate the initial context factory class : ");
            errorSB.append(properties.getProperty(Context.INITIAL_CONTEXT_FACTORY));
            errorSB.append(" : InstantiationException.");

            //Raise an exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (IllegalAccessException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create initial context : ");
            errorSB.append("can't instanciate the initial context factory class : ");
            errorSB.append(properties.getProperty(Context.INITIAL_CONTEXT_FACTORY));
            errorSB.append(" : IllegalAccessException.");

            //Raise an exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (ClassNotFoundException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create initial context : ");
            errorSB.append("can't load the initial context factory class : ");
            errorSB.append(properties.getProperty(Context.INITIAL_CONTEXT_FACTORY));
            errorSB.append(" : ClassNotFoundException.");

            //Raise an exception
            throw new ConnectionException(errorSB.toString(), ex);
        }

        return initialContext;
    }

    /**
     * Loads the Remote Interface of an EJB 2.x.
     *
     * @param info information on the EJB to call
     * @return the loaded interface
     * @throws ConnectionException if the interface can't be loaded
     */
    private Class<?> loadEjbHomeClass(final EjbConfiguration info) throws ConnectionException, ConfigurationException {
        //Retrieve the current classloader
        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        //Loads the Remote home class
        Class<?> clazz = null;
        try {
            clazz = cl.loadClass(info.getEjbElement().getEjbHomeInterface());
        } catch (ClassNotFoundException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't load EJB Remote Home Interface");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString(), ex);
        }

        return clazz;
    }

    /**
     * Calls (using reflection) the create method on an EJB 2.x remote home interface.
     *
     * @param home an EJB 2.x remote home interface
     * @return an EJB2.x stub
     * @throws ConnectionException is any problem occur while trying to call the create method
     */
    private Object createEjb2x(Object home) throws ConnectionException {
        //Try to load the create method of the EJB
        Method ejbCreateMethod = null;
        try {
            ejbCreateMethod = home.getClass().getMethod("create", new Class[]{});
        } catch (NoSuchMethodException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create an EJB stub from the given EJB RemoteHome interface ");
            errorSB.append("(" + home.getClass() + ") : ");
            errorSB.append("Your home interface must define a create() method without arguments : ");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (SecurityException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create an EJB stub from the given EJB RemoteHome interface ");
            errorSB.append("(" + home.getClass() + ") : ");
            errorSB.append("Security error : ");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString());
        }

        //Try to load the EJB
        Object ejb2Object = null;
        try {
            ejb2Object = ejbCreateMethod.invoke(home, new Object[]{});
        } catch (IllegalAccessException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create an EJB stub from the given EJB RemoteHome interface ");
            errorSB.append("(" + home.getClass() + ") : ");
            errorSB.append("IllegalAccess : ");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (IllegalArgumentException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create an EJB stub from the given EJB RemoteHome interface ");
            errorSB.append("(" + home.getClass() + ") : ");
            errorSB.append("IllegalArgument : ");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString(), ex);
        } catch (InvocationTargetException ex) {
            //Build the error message
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't create an EJB stub from the given EJB RemoteHome interface ");
            errorSB.append("(" + home.getClass() + ") : ");
            errorSB.append("InvocationTarget : ");
            errorSB.append(ex.getCause());

            //Raise a new Exception
            throw new ConnectionException(errorSB.toString(), ex);
        }

        return ejb2Object;
    }
}
