/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.service;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import org.ow2.petals.bc.ejb.exceptions.ConfigurationException;
import org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException;
import org.ow2.petals.component.framework.api.message.Exchange;
import java.lang.reflect.Method;
import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.ow2.petals.bc.ejb.exceptions.AuthenticationException;
import org.ow2.petals.databinding.jaxb.DataBindingException;
import org.w3c.dom.Document;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import org.ow2.petals.bc.ejb.EjbComponent;
import org.ow2.petals.bc.ejb.EjbConnectionPoolElement;
import org.ow2.petals.bc.ejb.configuration.EjbConfiguration;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.databinding.jaxb.service.Service;
import org.xml.sax.SAXException;

/**
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbService {

    //TODO : documentation
    private final Logger logger;
    /**
     * A reference onto the component that provides an EjbService.
     */
    private final EjbComponent component;

    /**
     * Default constructor.
     *
     * @param logger
     * @param ejbComponent
     */
    public EjbService(final Logger logger, final EjbComponent ejbComponent) {
        super();
        this.logger = logger;
        this.component = ejbComponent;
    }

    public void callEjb(final Provides provides, final Exchange exchange) throws JBIException {

        //Checks the MEP of the incoming message
        if (!(exchange.isInOutPattern() || exchange.isInOptionalOutPattern())) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Supported exchange pattern are IN-OUT and IN-OPTIONAL-OUT.");
            throw new JBIException(errorSB.toString());
        }

        //Check if the message has attachements
        if (!exchange.getInMessageAttachments().isEmpty()) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No attachement expected.");
            throw new JBIException(errorSB.toString());
        }

        //Retrieve the EJB holder using the given provides
        EjbConnectionPoolElement poolElement = this.retrieveEjbConnectionPoolElementFromProvides(provides);

        //Change the context class loader with the one used to register the SU
        ClassLoader suClassLoader = poolElement.getSuClassLoader();
        ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(suClassLoader);

        //Try to call the EJB
        LoginContext loginContext = null;
        try {

            //Retrieve the request message from the exchange
            Document requestDocument = EjbService.retrieveRequestDocumentFromExchange(exchange);

            //Retrieve the operation from the exchange
            String operation = this.retrieveOperationFromExchange(exchange);

            //Check if a security subject exists in MessageExchange to login
            Subject subject = exchange.getInMessage().getSecuritySubject();
            if (subject != null) {
                loginContext = login(poolElement.getEjbConfiguration(), subject);
            }

            //Call ejb
            Document responseDocument = null;
            responseDocument = this.callEjbUsingJAXB(poolElement, operation, requestDocument);

            if (responseDocument != null) {
                exchange.setOutMessageContent(responseDocument);
            }

        } finally {
            // reset old classloader
            Thread.currentThread().setContextClassLoader(oldCL);

            // Logout
            if (loginContext != null) {
                this.logout(loginContext);
            }
        }

    }

    private String retrieveOperationFromExchange(final Exchange exchange) throws JBIException {
        //Retrieve the operation
        final QName operation = exchange.getOperation();
        String operationName = null;

        //Retrieve the operation name
        if (operation != null) {
            operationName = operation.getLocalPart();
        }

        //If no operation is set on the message exchnge
        if (operationName == null || "".equals(operationName)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No operation found on the message exchange.");
            throw new JBIException(errorSB.toString());
        }

        return operationName;
    }

    private static Document retrieveRequestDocumentFromExchange(final Exchange exchange) throws JBIException {
        Document requestDocument = null;
        try {
            //Checking the message content
            requestDocument = exchange.getInMessageContentAsDocument();
        } catch (MessagingException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retireve message content for exchange : ");
            errorSB.append(exchange.getExchangeId());
            errorSB.append(" : Messaging error : ");
            throw new JBIException(errorSB.toString(), ex);
        }

        return requestDocument;
    }

    private EjbConnectionPoolElement retrieveEjbConnectionPoolElementFromProvides(final Provides provides) throws JBIException {
        EjbConnectionPoolElement poolElement = null;
        try {
            poolElement = this.component.getEjbConnectionPool().getEjb(provides);
        } catch (EjbConnectionPoolException ex) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retireve the EJB from the given provides : ");
            errorSB.append(provides.toString());
            throw new JBIException(errorSB.toString(), ex);
        }

        return poolElement;
    }

    /**
     * Performs the EJB call.
     *
     * @param poolElement
     *      the poolElement that reference the EJB to call
     * @param operation
     *      the operation to call on the EJB
     * @param requestDocument
     *      the operation parameters
     * @return
     *      the response
     * @throws javax.jbi.JBIException
     *      if any error occurs while performing the EJB call
     */
    private Document callEjbUsingJAXB(final EjbConnectionPoolElement poolElement, final String operation, final Document requestDocument) throws JBIException {

        //Try to call the EJB
        try {

            // Get the JbiService object to Marshall/Unmarshall the message content
            final Source messageRequest = new DOMSource(requestDocument);

            //Retrieve the databinding service
            final Service jbiService = poolElement.getDatabindingService();

            Document messageResult = null;
            Object[] methodParamValue = null;
            Method methodToInvoke = null;
            Object methodResult = null;

            //Create the method parameters from the incoming message
            methodParamValue = jbiService.getServiceUnmarshaller().unmarshallMethodParam(
                    jbiService.getOperationByName(operation), messageRequest);

            //Call the EJB method
            if (methodParamValue != null) {
                methodToInvoke = poolElement.getEjbObject().getClass().getMethod(operation,
                        jbiService.getOperationByName(operation).getParameterTypes());

                methodResult = methodToInvoke.invoke(poolElement.getEjbObject(), methodParamValue);
            }

            //Marhsall the method result to create the output message content
            if (methodResult != null) {

                messageResult = jbiService.getServiceMarshaller().marshallMethodResult(
                        jbiService.getOperationByName(operation), new Object[]{methodResult});
            }


            return messageResult;

        } catch (IllegalAccessException ex) {

            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't call : ");
            errorSB.append(operation);
            errorSB.append(" : Illegal Access error : ");
            throw new JBIException(errorSB.toString(), ex);

        } catch (InvocationTargetException ex) {

            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't call : ");
            errorSB.append(operation);
            errorSB.append(" : Invocation Target error : ");
            throw new JBIException(errorSB.toString(), ex);

        } catch (NoSuchMethodException ex) {

            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't call : ");
            errorSB.append(operation);
            errorSB.append(" : No Such Method error : ");
            throw new JBIException(errorSB.toString(), ex);

        } catch (DataBindingException ex) {

            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't unmarshall request or can't marshall response : ");
            errorSB.append(operation);
            errorSB.append(" : DataBinding error : ");
            throw new JBIException(errorSB.toString(), ex);

        }
    }

    private LoginContext login(final EjbConfiguration ejbConfiguration, final Subject subject) throws JBIException {
        try {
            //Holds the login context (what a surprise :)
            LoginContext loginContext = null;

            //Retrieve the security name
            final String securityName = ejbConfiguration.getSecurityElement().getSecurityName();

            //If there's a given security name
            if (securityName != null) {

                //Trying to retrieve a valid LoginContext from it
                try {
                    loginContext = new LoginContext(securityName, subject);
                } catch (LoginException ex) {
                    StringBuilder errorSB = new StringBuilder();
                    errorSB.append("Can't instanciate login context : ");
                    throw new AuthenticationException(errorSB.toString(), ex);
                }

                //Trying to perform login
                if (loginContext != null) {
                    try {
                        loginContext.login();
                    } catch (LoginException ex) {
                        StringBuilder errorSB = new StringBuilder();
                        errorSB.append("Can't perform login operation : ");
                        throw new AuthenticationException(errorSB.toString(), ex);
                    }
                }
            }

            return loginContext;

        } catch (ConfigurationException ex) {
            throw new JBIException(ex);
        } catch (AuthenticationException ex) {
            throw new JBIException(ex);
        }
    }

    private void logout(LoginContext loginContext) throws JBIException {
        try {
            try {
                loginContext.logout();
            } catch (LoginException ex) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't perform logout operation : ");
                throw new AuthenticationException(errorSB.toString(), ex);
            }
        } catch (AuthenticationException ex) {
            throw new JBIException(ex);
        }
    }
}
